/***********************************
CRITICAL PRELIMINARY REQUIREMENTS
1. Download MIXREGLS.zip: https://www.jstatsoft.org/article/view/v052i12
2. Unzip MIXREGLS.zip folder (critical files in the folder: mixregls.def and MixRegLSb.exe)
3. Note directory for unzipped MIXREGLS folder
4. As an alternative, you could try NLMIXED, too--see lines 529-697 below
************************************/

***************************************************************
***            Define MACRO Variable Values Below           ***
***      Use Calculate_Values.xlsx to Determine Values      ***
***************************************************************;

*Folder Directory for (unzipped) MIXREGLS;
	%LET MIXREGLS = E:\MIXREGLS;
*Individuals; 
	%LET N = 50;
*Repeated Occasions; 
	%LET ni = 20;
*Location-Model Fixed Intercept (Outcome Scale); 
	%LET beta0 = 0;
*Location-Model Random Intercept Variance (Outcome Scale); 
	%LET b0var = 2;
*Scale-Model Fixed Intercept (Log Scale); 
	%LET tau0 = 0.643;
*Scale-Model Random Intercept Variance; 
	%LET t0var = 0.10;
*Correlation between Location- and Scale-Model Random Intercepts;
	%LET b0t0corr = 0.30;
*Scale-Model Fixed Predictor W_i Effect;
	%LET tau1 = 0.133;
*Pseudo-R2;
	%LET R2 = 0.10;
*Is the Predictor W_i Continuous? (1=Yes, 0=No);
	%LET Continuous = 1;
		*Mean of Predictor W_i (=0 if W_i is continuous);
			%LET Wmean = 0;
		*Variance of Continuous Predictor W_i;
			*Does not apply to binary predictor, but must have 
		     non-null, non-zero value after equal sign for program 
			 to run;
			%LET Wvar = 0.563;
*Number of Replications for Power Analysis;
	%LET Nreps = 1000;

************************************************
***      !DO NOT CHANGE ANYTHING BELOW!      ***
***   Create Data, Run MIXREGLS, Get Power   ***
************************************************;

*Define MACRO to remove empty MIXREGLS results - No Predictor;
%MACRO Fill_NP;
	%LET name_list = Neg2LL_NP 
					 beta0_NP 
					 b0var_NP
					 tau0_NP 
					 RLonWS_NP
					 t0sd_NP;
	%LOCAL i next_name;
	%LET i=1;
	%DO %WHILE (%SCAN(&name_list, &i.) NE );
	   %LET next_name = %SCAN(&name_list, &i.);
			PROC SORT DATA=Results; BY DESCENDING &next_name.; RUN;
			DATA Results; SET Results;
		   		RETAIN var;
			   	IF NOT MISSING(&next_name.) THEN var=&next_name.;
				   ELSE &next_name.=var;
				   DROP var;
			RUN;
		 %LET i = %EVAL(&i. + 1);
	%END;
	PROC SORT DATA=Results NODUPKEY OUT=Results_NP(DROP=line row); BY Neg2LL_NP; RUN;
%MEND Fill_NP;

*Define MACRO to remove empty MIXREGLS results - Include Predictor;
%MACRO Fill_YP;
	%LET name_list = Neg2LL_YP 
					 beta0_YP 
					 b0var_YP
					 tau0_YP
					 tau1_YP
					 tau1_SE_YP
					 RLonWS_YP
					 t0sd_YP;
	%LOCAL i next_name;
	%LET i=1;
	%DO %WHILE (%SCAN(&name_list, &i.) NE );
	   %LET next_name = %SCAN(&name_list, &i.);
			PROC SORT DATA=Results; BY DESCENDING &next_name.; RUN;
			DATA Results; SET Results;
		   		RETAIN var;
			   	IF NOT MISSING(&next_name.) THEN var=&next_name.;
				   ELSE &next_name.=var;
				   DROP var;
			RUN;
		 %LET i = %EVAL(&i. + 1);
	%END;
	PROC SORT DATA=Results NODUPKEY OUT=Results_YP(DROP=line row); BY Neg2LL_YP; RUN;
%MEND Fill_YP;

%MACRO Power_Predict;

	*Turn Off SAS Log;
	PROC PRINTTO LOG=junk; RUN;

	*Delete Power_Predict Dataset from Previous Simulation;
	PROC DATASETS LIB=work NOLIST KILL; RUN; QUIT;

	*Calculate SMRI for simulation;
	DATA _NULL_;
		t0var_sim = &t0var.*(1 - &R2.);
		CALL SYMPUTX ("t0var_sim",t0var_sim);
	RUN;

	%DO Rep = 1 %TO &Nreps.;

		***********************
		***   Create Data   ***
		***********************;

		*Create Data for the Mixed-Effects Location-Scale Model;
		DATA MELS;
			KEEP Rep PersonID MeasureID W Y;
			Rep = &Rep.;
			rho = &b0t0corr.;
		    DO PersonID = 1 TO &N.;
				*Generate Standard Normal Variates with Specified Correlation;
				b0i = RAND("NORMAL",0,SQRT(1));
		    	t0i = rho*b0i + SQRT(1 - rho**2)*RAND("NORMAL",0,SQRT(1));
				*Transform Intercepts to have (Prior) Mean of 0 with Specified Variance;
		        b0i = 0 + SQRT(&b0var.)*b0i;
		        t0i = 0 + SQRT(&t0var_sim.)*t0i;
				*Generate Predictor Value;
				IF &Continuous. = 0 THEN 
					DO;
						W = RAND("BERNOULLI",&Wmean.);
					END;
				IF &Continuous. = 1 THEN 
					DO; 
						W = RAND("NORMAL",&Wmean.,SQRT(&Wvar.));
					END;
				*Create Repeated Measures;
				DO MeasureID = 1 TO &ni.;
					*Create Heterogeneous Residual Variances;
					evar_i = EXP((&tau0. + t0i) + &tau1.*W);
					e_ti = RAND("NORMAL",0,SQRT(evar_i));
					*Create Outcome Variable;
					Y = (&beta0. + b0i) + e_ti;
					OUTPUT;
				END;
			END;
		RUN;

		***********************************************
		***   Estimate Models & Aggregate Results   ***
		***********************************************;

		*Write out the new dataset for MIXREGLS;
		DATA _NULL_; SET MELS;
			FILE "&MIXREGLS.\data.dat";
			PUT Rep PersonID MeasureID W Y;
		RUN;

		*No Location- and Scale-Model Intercept Correlation;
		%IF (%SYSEVALF(&b0t0corr. = 0)) %THEN %DO;

			*Create MIXREGLS Script - No Predictor;
			DATA _NULL_;
				FILE "&MIXREGLS.\Model.def";
				PUT "Analysis of Y - 15 quad pts"
				/ "BS and WS variance model with random scale"
				/ "Data.dat"
				/ "Results.out"
				/ "Model.def"
				/ "5 0 0 0 0 0 0 0.00001 15 1 200 0 0 0"
				/ "2 5"
				/ "Y"; 
			RUN;

			*Run MIXREGLS;
			OPTIONS XSYNC NOXWAIT;
			X "cd &MIXREGLS."; RUN;
			X "cd &MIXREGLS.
			   & del mixregls.def & copy Model.def mixregls.def & mixreglsb";
			RUN;

			*Import MIXREGLS output into SAS;
			DATA Results;
				INFILE "&MIXREGLS.\MIXREGLS.EST" DSD TRUNCOVER LRECL=9000;
				INPUT line $200.;
			RUN;

			*Locate Relevant Results;
			DATA Results; SET Results;
				*NP = Model Did Not Include Scale-Model Predictor W;
				*LM = Location Model;
				*SM = Scale Model for the Residual Variance;
			    Rep = &Rep.;
				Row = _N_;
				*Model Misfit for Comparison;
				IF Row = 6 THEN Neg2LL_NP = INPUT(SCAN(line,1," "), BEST15.);
				*Location-Model Fixed Intercept;
				IF Row = 7  THEN beta0_NP = INPUT(SCAN(line,1," "), BEST15.);
				*Location-Model Random Intercept Variance;
				IF Row = 8  THEN b0var_NP = INPUT(SCAN(line,1," "), BEST15.);
				*Scale-Model Fixed Intercept (Unit-Specific with Scale-Model Random Intercept);
				IF Row = 9  THEN tau0_NP = INPUT(SCAN(line,1," "), BEST15.);
				*Scale-Model Random Intercept Standard Deviation;
				IF Row = 10 THEN t0sd_NP = INPUT(SCAN(line,1," "), BEST15.);
			RUN;

			*Call Fill MACRO to remove empty cells;
			%Fill_NP;

			*Create MIXREGLS script - Include predictor;
			DATA _NULL_;
				FILE "&MIXREGLS.\Model.def";
				PUT "Analysis of Y - 15 quad pts"
				/ "BS and WS variance model with random scale"
				/ "Data.dat"
				/ "Results.out"
				/ "Model.def"
				/ "5 0 0 1 0 0 0 0.00001 15 1 200 0 0 0"
				/ "2 5"
				/ "4" 
				/ "Y"
				/ "W"; 
			RUN;

			*Run MIXREGLS - Include Predictor;
			OPTIONS XSYNC NOXWAIT;
			X "cd &MIXREGLS."; RUN;
			X "cd &MIXREGLS.
			   & del mixregls.def & copy Model.def mixregls.def & mixreglsb";
			RUN;

			*Import MIXREGLS output into SAS - Include Predictor;
			DATA Results;
				INFILE "&MIXREGLS.\MIXREGLS.EST" DSD TRUNCOVER LRECL=9000;
				INPUT line $200.;
			RUN;

			*Locate Relevant Results - Include Predictor;
			DATA Results; SET Results;
				*YP = Model Did Include Scale-Model Predictor W;
				*LM = Location Model;
				*SM = Scale Model for the Residual Variance;
			    Rep = &Rep.;
				Row = _N_;
				*Model Misfit for Comparison;
				IF Row = 11 THEN Neg2LL_YP = INPUT(SCAN(line,1," "), BEST15.);
				*Location-Model Fixed Intercept;
				IF Row = 12 THEN beta0_YP = INPUT(SCAN(line,1," "), BEST15.);
				*Location-Model Random Intercept Variance;
				IF Row = 13 THEN b0var_YP = INPUT(SCAN(line,1," "), BEST15.);
				*Scale-Model Fixed Intercept (Unit-Specific with Scale-Model Random Intercept);
				IF Row = 14 THEN tau0_YP = INPUT(SCAN(line,1," "), BEST15.);
				*Scale-Model Fixed Effect of Predictor W (unit-specific);
				IF Row = 14 THEN tau1_YP = INPUT(SCAN(line,2," "), BEST15.);
				IF Row = 16 THEN tau1_SE_YP = INPUT(SCAN(line,4," "), BEST15.);
				*Scale-Model Random Intercept Standard Deviation;
				IF Row = 15 THEN t0sd_YP = INPUT(SCAN(line,1," "), BEST15.);
			RUN;

			*Call Fill MACRO to remove empty cells;
			%Fill_YP;

			*Merge Results from No Predictor and Include Predictor Models;
			DATA Results; MERGE Results_NP Results_YP; BY Rep; RUN;

			*Determine Significance of Scale-Model Predictor W;
			DATA Results; 
			RETAIN Rep
				   beta0_NP 
				   b0var_NP b0var_NP_DS
				   tau0_NP 
				   t0sd_NP t0var_NP
				   beta0_YP 
				   b0var_YP b0var_YP_DS
				   tau0_YP
				   tau1_YP tau1_SE_YP tau1_Lower_YP tau1_Upper_YP tau1_SS
				   t0sd_YP t0var_YP
				   PseudoR2
				   Neg2LL_NP Neg2LL_YP;
			SET Results;
				*Location-Model Random Intercpet Variance;
				*DS = Data Scale;
				b0var_NP_DS = EXP(b0var_NP);
				b0var_YP_DS = EXP(b0var_YP);
				*Scale-Model Random Intercept Variance (Log Scale);
			    t0var_NP = t0sd_NP**2;
				t0var_YP = t0sd_YP**2;
				*Significance of Scale-Model Random Intercept Variance
				 using Likelihood Ratio Test (Mixture Chi-Square);
				*SS = Statistically Significant at p < .05 by 95% CI;
				tau1_Lower_YP = tau1_YP - 1.96*(tau1_SE_YP);
				tau1_Upper_YP = tau1_YP + 1.96*(tau1_SE_YP);
				IF (tau1_Lower_YP > 0 OR tau1_Upper_YP < 0) THEN tau1_SS = 1; ELSE tau1_SS = 0;
				*Pseudo-R2;
				PseudoR2 = ((t0var_NP - t0var_YP) / t0var_NP);
				*Variable Labels;
				LABEL b0var_NP_DS="LM RI Variance (Outcome Scale) - No Predictor"
					  b0var_YP_DS="LM RI Variance (Outcome Scale) - Yes Predictor"
					  beta0_NP="LM Fixed Intercept - No Predictor"
					  beta0_YP="LM Fixed Intercept - Yes Predictor"
					  b0var_NP="LM RI Variance (Log Scale) - No Predictor"
					  b0var_YP="LM RI Variance (Log Scale) - Yes Predictor"
					  b0var_NP_DS="LM RI Variance (Outcome Scale) - No Predictor"
					  b0var_YP_DS="LM RI Variance (Outcome Scale) - Yes Predictor"
					  tau0_NP="SM Fixed Intercept (Log Scale) - No Predictor"
					  tau0_YP="SM Fixed Intercept (Log Scale) - Yes Predictor"
					  t0var_NP="SM RI Variance (Log Scale) - No Predictor"
					  t0var_YP="SM RI Variance (Log Scale) - Yes Predictor"
					  tau1_YP="SM Fixed Effect of Predictor W (Log Scale)"
					  tau1_SS="Scale-Model Predictor W Significant: 0=No, 1=Yes"
					  PseudoR2="Proportion Reduction SM RI Variance from Predictor W";
			RUN;

		%END;

		*Yes Location- and Scale-Model Intercept Correlation;
		%IF (%SYSEVALF(&b0t0corr. NE 0)) %THEN %DO;

			*Create MIXREGLS Script - No Predictor;
			DATA _NULL_;
				FILE "&MIXREGLS.\Model.def";
				PUT "Analysis of Y - 15 quad pts"
				/ "BS and WS variance model with random scale"
				/ "Data.dat"
				/ "Results.out"
				/ "Model.def"
				/ "5 0 0 0 0 0 0 0.00001 15 1 200 0 0 1"
				/ "2 5"
				/ "Y"; 
			RUN;

			*Run MIXREGLS;
			OPTIONS XSYNC NOXWAIT;
			X "cd &MIXREGLS."; RUN;
			X "cd &MIXREGLS.
			   & del mixregls.def & copy Model.def mixregls.def & mixreglsb";
			RUN;

			*Import MIXREGLS output into SAS;
			DATA Results;
				INFILE "&MIXREGLS.\MIXREGLS.EST" DSD TRUNCOVER LRECL=9000;
				INPUT line $200.;
			RUN;

			*Locate Relevant Results;
			DATA Results; SET Results;
				*NP = Model Did Not Include Scale-Model Predictor W;
				*LM = Location Model;
				*SM = Scale Model for the Residual Variance;
			    Rep = &Rep.;
				Row = _N_;
				*Model Misfit for Comparison;
				IF Row = 6 THEN Neg2LL_NP = INPUT(SCAN(line,1," "), BEST15.);
				*Location-Model Fixed Intercept;
				IF Row = 7  THEN beta0_NP = INPUT(SCAN(line,1," "), BEST15.);
				*Location-Model Random Intercept Variance;
				IF Row = 8  THEN b0var_NP = INPUT(SCAN(line,1," "), BEST15.);
				*Scale-Model Fixed Intercept (Unit-Specific with Scale-Model Random Intercept);
				IF Row = 9  THEN tau0_NP = INPUT(SCAN(line,1," "), BEST15.);
				*Linear Effect between Location- and Scale-Model Random Intercepts;
				IF Row = 10 THEN RLonWS_NP = INPUT(SCAN(line,1," "), BEST15.);
				*Scale-Model Random Intercept Standard Deviation;
				IF Row = 10 THEN t0sd_NP = INPUT(SCAN(line,2," "), BEST15.);
			RUN;

			*Call Fill MACRO to remove empty cells;
			%Fill_NP;

			*Create MIXREGLS script - Include predictor;
			DATA _NULL_;
				FILE "&MIXREGLS.\Model.def";
				PUT "Analysis of Y - 15 quad pts"
				/ "BS and WS variance model with random scale"
				/ "Data.dat"
				/ "Results.out"
				/ "Model.def"
				/ "5 0 0 1 0 0 0 0.00001 15 1 200 0 0 1"
				/ "2 5"
				/ "4" 
				/ "Y"
				/ "W"; 
			RUN;

			*Run MIXREGLS - Include Predictor;
			OPTIONS XSYNC NOXWAIT;
			X "cd &MIXREGLS."; RUN;
			X "cd &MIXREGLS.
			   & del mixregls.def & copy Model.def mixregls.def & mixreglsb";
			RUN;

			*Import MIXREGLS output into SAS - Include Predictor;
			DATA Results;
				INFILE "&MIXREGLS.\MIXREGLS.EST" DSD TRUNCOVER LRECL=9000;
				INPUT line $200.;
			RUN;

			*Locate Relevant Results - Include Predictor;
			DATA Results; SET Results;
				*YP = Model Did Include Scale-Model Predictor W;
				*LM = Location Model;
				*SM = Scale Model for the Residual Variance;
			    Rep = &Rep.;
				Row = _N_;
				*Model Misfit for Comparison;
				IF Row = 11 THEN Neg2LL_YP = INPUT(SCAN(line,1," "), BEST15.);
				*Location-Model Fixed Intercept;
				IF Row = 12 THEN beta0_YP = INPUT(SCAN(line,1," "), BEST15.);
				*Location-Model Random Intercept Variance;
				IF Row = 13 THEN b0var_YP = INPUT(SCAN(line,1," "), BEST15.);
				*Scale-Model Fixed Intercept (Unit-Specific with Scale-Model Random Intercept);
				IF Row = 14 THEN tau0_YP = INPUT(SCAN(line,1," "), BEST15.);
				*Scale-Model Fixed Effect of Predictor W (unit-specific);
				IF Row = 14 THEN tau1_YP = INPUT(SCAN(line,2," "), BEST15.);
				IF Row = 16 THEN tau1_SE_YP = INPUT(SCAN(line,4," "), BEST15.);
				*Linear Effect between Location- and Scale-Model Random Intercepts;
				IF Row = 15 THEN RLonWS_YP = INPUT(SCAN(line,1," "), BEST15.);
				*Scale-Model Random Intercept Standard Deviation;
				IF Row = 15 THEN t0sd_YP = INPUT(SCAN(line,2," "), BEST15.);
			RUN;

			*Call Fill MACRO to remove empty cells;
			%Fill_YP;

			*Merge Results from No Predictor and Include Predictor Models;
			DATA Results; MERGE Results_NP Results_YP; BY Rep; RUN;

			*Determine Significance of Scale-Model Predictor W;
			DATA Results; 
			RETAIN Rep
				   beta0_NP 
				   b0var_NP b0var_NP_DS
				   tau0_NP 
				   RLonWS_NP b0t0cov_NP b0t0corr_NP
				   t0sd_NP t0var_NP
				   beta0_YP 
				   b0var_YP b0var_YP_DS
				   tau0_YP
				   tau1_YP tau1_SE_YP tau1_Lower_YP tau1_Upper_YP tau1_SS
				   RLonWS_YP b0t0cov_YP b0t0corr_YP
				   t0sd_YP t0var_YP
				   PseudoR2
				   Neg2LL_NP Neg2LL_YP;
			SET Results;
				*Location-Model Random Intercpet Variance;
				*DS = Data Scale;
				b0var_NP_DS = EXP(b0var_NP);
				b0var_YP_DS = EXP(b0var_YP);
				*Scale-Model Random Intercept Variance (Log Scale);
			    t0var_NP =  (RLonWS_NP**2 + t0sd_NP**2);
				t0var_YP =  (RLonWS_YP**2 + t0sd_YP**2);
				*Correlation between Location- and Scale-Model Random Intercepts;
				b0t0cov_NP = RLonWS_NP*SQRT(b0var_NP_DS);
				b0t0corr_NP = b0t0cov_NP / (SQRT(b0var_NP_DS*t0var_NP));
				b0t0cov_YP = RLonWS_YP*SQRT(b0var_YP_DS);
				b0t0corr_YP = b0t0cov_YP / (SQRT(b0var_YP_DS*t0var_YP));
				*Significance of Scale-Model Random Intercept Variance
				 using Likelihood Ratio Test (Mixture Chi-Square);
				*SS = Statistically Significant at p < .05 by 95% CI;
				tau1_Lower_YP = tau1_YP - 1.96*(tau1_SE_YP);
				tau1_Upper_YP = tau1_YP + 1.96*(tau1_SE_YP);
				IF (tau1_Lower_YP > 0 OR tau1_Upper_YP < 0) THEN tau1_SS = 1; ELSE tau1_SS = 0;
				*Pseudo-R2;
				PseudoR2 = ((t0var_NP - t0var_YP) / t0var_NP);
				*Variable Labels;
				LABEL beta0_NP="LM Fixed Intercept (Outcome Scale) - No Predictor"
					  beta0_YP="LM Fixed Intercept (Outcome Scale) - Yes Predictor"
					  b0var_NP="LM RI Variance (Log Scale) - No Predictor" 
					  b0var_YP="LM RI Variance (Log Scale) - Yes Predictor" 
					  b0var_NP_DS="LM RI Variance (Outcome Scale) - No Predictor"
					  b0var_YP_DS="LM RI Variance (Outcome Scale) - Yes Predictor"
					  tau0_NP="SM Fixed Intercept (Log Scale) - No Predictor"
					  tau0_YP="SM Fixed Intercept (Log Scale) - Yes Predictor"
					  t0var_NP="SM RI Variance - No Predictor"
					  t0var_YP="SM RI Variance - Yes Predictor"
					  b0t0corr_NP="Correlation b/w LM and SM RI - No Predictor"
					  b0t0corr_YP="Correlation b/w LM and SM RI - Yes Predictor"
					  tau1_YP="SM Fixed Effect of Predictor W (Log Scale)"
					  tau1_SS="Scale-Model Predictor W Significant: 0=No, 1=Yes"
					  PseudoR2="Proportion Reduction SM RI Variance from Predictor W";
			RUN;

		%END;

		*Append Results;
		PROC APPEND BASE=Power_Predict DATA=Results; RUN;

	%END;

	*Turn On SAS Log;
	PROC PRINTTO; RUN;

	%IF (%SYSEVALF(&b0t0corr. = 0)) %THEN %DO;
		TITLE "Parameter Recovery for &N. Individuals with &ni. Occasions";
		PROC MEANS DATA=Power_Predict N MEAN CLM P25 P50 P75 MAXDEC=3; 
			WHERE t0sd_NP > 0 AND t0sd_NP > 0;
			VAR beta0_YP b0var_YP_DS
				tau0_YP tau1_YP
	 			t0var_NP t0var_YP PseudoR2; 
		RUN; TITLE;
	%END;

	%IF (%SYSEVALF(&b0t0corr. NE 0)) %THEN %DO;
		TITLE "Parameter Recovery for &N. Individuals with &ni. Occasions";
		PROC MEANS DATA=Power_Predict N MEAN CLM P25 P50 P75 MAXDEC=3; 
			WHERE t0sd_NP > 0 AND t0sd_NP > 0;
			VAR beta0_YP b0var_YP_DS 
				tau0_YP tau1_YP
	 			t0var_NP t0var_YP PseudoR2
				b0t0corr_YP; 
		RUN; TITLE;
	%END;

	TITLE "Estimated Power for &N. Individuals with &ni. Occasions";
	PROC FREQ DATA=Power_Predict;
		WHERE t0sd_NP > 0; 
		TABLES tau1_SS / BINOMIAL(LEVEL="1");
	RUN; TITLE;

%MEND Power_Predict;

*Call Power MACRO;
%Power_Predict;
/*
**************************************
***   Power Analysis via NLMIXED   ***
**************************************;

*Calculate SMRI for simulation;
DATA _NULL_;
	t0var_sim = &t0var.*(1 - &R2.);
	CALL SYMPUTX ("t0var_sim",t0var_sim);
RUN;

*Create Data for the Mixed-Effects Location-Scale Model;
DATA MELS_NLMIXED;
	KEEP Rep PersonID MeasureID W Y;
	rho = &b0t0corr.;
	DO rep = 1 TO &Nreps.;
	    DO PersonID = 1 TO &N;
			*Generate Standard Normal Variates with Specified Correlation;
			b0i = RAND("NORMAL",0,SQRT(1));
	    	t0i = rho*b0i + SQRT(1 - rho**2)*RAND("NORMAL",0,SQRT(1));
			*Transform Intercepts to have (Prior) Mean of 0 with Specified Variance;
	        b0i = 0 + SQRT(&b0var.)*b0i;
	        t0i = 0 + SQRT(&t0var_sim.)*t0i;
			*Generate Predictor Value;
			IF &Continuous. = 0 THEN 
				DO;
					W = RAND("BERNOULLI",&Wmean.);
				END;
			IF &Continuous. = 1 THEN 
				DO; 
					W = RAND("NORMAL",&Wmean.,SQRT(&Wvar.));
				END;
			*Create Repeated Measures;
			DO MeasureID = 1 TO &ni.;
				*Create Heterogeneous Residual Variances;
				evar_i = EXP((&tau0. + t0i) + &tau1.*W);
				e_ti = RAND("NORMAL",0,SQRT(evar_i));
				*Create Outcome Variable;
				Y = (&beta0. + b0i) + e_ti;
				OUTPUT;
			END;
		END;
	END;
RUN;

*Turn off all output;
ODS HTML CLOSE; ODS GRAPHICS OFF; ODS EXCLUDE ALL; ODS NORESULTS;

*Set LM-SM RI covariance;
DATA _NULL_;
	b0t0cov=&b0t0corr.*(SQRT(&b0var.*&t0var.));
	CALL SYMPUTX ("b0t0cov",b0t0cov);
RUN;

*I owe a huge thanks to Dale McLerran for helping with the NLMIXED program below.
 He took time to share his expertise on scale-model random intercept variance 
 estimation difficultes. His suggestions for programming the NLMIXED code below 
 can be found here: https://stackoverflow.com/a/46475970/3798875
 Also, if the program blows up (it will...), check the log for the rep that blew up and
 exclude that rep using the WHERE statement. For example, if rep 5 blows up, include
 WHERE Rep NE 5 in between the NLMIXED and BY statements. Keep adding reps to 
 the WHERE statement as needed;

TITLE1 "Unconditional Mixed-Effects Location-Scale Model";
TITLE2 "Use Grid Search on PARMS Statement";
PROC NLMIXED DATA=MELS_NLMIXED GCONV=1E-12 QPOINTS=15;
	BY Rep;
	*Starting Values;
	PARMS beta0=&beta0.
		  tau0=&tau0.
		  ln_sd_b0var=%SYSFUNC(LOG(%SYSFUNC(SQRT(&b0var.))))
		  b0t0cov=&b0t0cov.
		  ln_sd_t0var=-5 TO 1 BY 0.05; *Change search parameters as needed!;
	*Location Model;
	mu = beta0 + b0i;
	*Scale Model for the Residual Variance;
	evar = EXP(tau0 + t0i);
	MODEL Y ~ NORMAL(mu,evar);
	RANDOM b0i t0i ~ NORMAL([0,0],
                            [exp(2*ln_sd_b0var), b0t0cov,
                             exp(2*ln_sd_t0var)]) SUBJECT=PersonID;
	ODS OUTPUT AdditionalEstimates=Est;
	ESTIMATE "Scale-Model RI Variance" EXP(2*ln_sd_t0var);
RUN; TITLE1; TITLE2;

*Get Scale-Model RI Variance;
DATA Est; SET Est; RENAME Estimate=t0var1; KEEP Rep Estimate; RUN;

*If the program below blows up (it will...), check the log for the rep that blew up
 and exclude that rep using the WHERE statement. For example, if rep 5 blows up, 
 include WHERE Rep NE 5 in between the NLMIXED and BY statements. Keep adding reps  
 to the WHERE statement as needed;

PROC NLMIXED DATA=MELS_NLMIXED GCONV=1E-12 QPOINTS=15;
	BY Rep;
	*Starting Values;
	PARMS beta0=&beta0.
		  tau0=&tau0.
		  tau1=&tau1.
		  ln_sd_b0var=%SYSFUNC(LOG(%SYSFUNC(SQRT(&b0var.))))
		  b0t0cov=&b0t0cov.
		  ln_sd_t0var=-5 TO 1 BY 0.05; *Change search parameters as needed!;
	*Location Model;
	mu = beta0 + b0i;
	*Scale Model for the Residual Variance;
	evar_i = EXP((tau0 + t0i) + tau1*W);
	MODEL Y ~ NORMAL(mu,evar_i);
	RANDOM b0i t0i ~ NORMAL([0,0],
                            [exp(2*ln_sd_b0var), b0t0cov,
                             exp(2*ln_sd_t0var)]) SUBJECT=PersonID;
	ODS OUTPUT AdditionalEstimates=Est2;
	ESTIMATE "Location-Model Fixed Intercept - Outcome Scale" beta0;
	ESTIMATE "Location-Model RI Variance - Outcome Scale" EXP(2*ln_sd_b0var);
	ESTIMATE "Scale-Model Fixed Intercept - Log Scale" tau0;
	ESTIMATE "Scale-Model RI Variance" EXP(2*ln_sd_t0var);
	ESTIMATE "Scale-Model Fixed Effect of W" tau1; 
	ESTIMATE "Location-Scale RI Covariance" b0t0cov;
	ESTIMATE "Location-Scale RI Correlation" b0t0cov / (SQRT(EXP(2*ln_sd_b0var))*SQRT(EXP(2*ln_sd_t0var)));
RUN; TITLE1; TITLE2;

*Get estimated parameters;
DATA Est3; SET Est2; KEEP Rep Estimate; RUN;
PROC TRANSPOSE DATA=Est3 OUT=Est_wide; BY Rep; VAR Estimate; RUN;
DATA Est_wide; SET Est_wide;
	IF &b0t0corr. = 0 THEN corr = 0; ELSE corr = 1; 
	RENAME COL1=beta0 COL2=b0var COL3=tau0 COL4=t0var2 COL5=tau1 COL6=b0t0cov COL7=b0t0corr;
	DROP _NAME_;
RUN;

*Store Minimum and Maximum W to be used to calculate heterogenous ICC;
PROC MEANS DATA=MELS_NLMIXED MIN MAX NOPRINT; BY Rep; VAR W; OUTPUT OUT=W MIN= MAX= / AUTONAME; RUN;

*Calculate hetergeneous ICC and PseudoR2;
DATA Est_wide; 
	MERGE Est_wide Est W; 
	BY Rep; 
	ICCminW = b0var / (b0var + EXP(tau0 + tau1*W_min + 0.5*(t0var2)));
	ICCmaxW = b0var / (b0var + EXP(tau0 + tau1*W_max + 0.5*(t0var2)));
	PseudoR2 = (t0var1 - t0var2) / t0var1;
	DROP _TYPE_ _FREQ_;
RUN;

*Get statistical significance of tau1;
DATA Est4; SET Est2; 
	IF Label="Scale-Model Fixed Effect of W";
	IF Probt < 0.05 THEN tau1sig = "Yes"; ELSE tau1sig = "No";
	KEEP Rep tau1sig;
RUN;

*Merge all data;
DATA Power_Predict_NLMIXED; MERGE Est_wide Est4; BY Rep; DROP Descr; RUN;

*Turn on all output;
ODS HTML; ODS GRAPHICS ON; ODS EXCLUDE NONE; ODS RESULTS;

TITLE "Parameter Recovery";
PROC MEANS DATA=Power_Predict_NLMIXED N MEAN CLM P25 P50 P75 MAXDEC=3;
	WHERE Corr = 1;
	VAR beta0 b0var tau0 t0var2 tau1 b0t0corr ICCminW ICCmaxW PseudoR2 t0var1; 
RUN; TITLE;

TITLE "Parameter Recovery";
PROC MEANS DATA=Power_Predict_NLMIXED N MEAN CLM P25 P50 P75 MAXDEC=3;
	WHERE Corr = 0;
	VAR beta0 b0var tau0 t0var2 tau1 ICCminW ICCmaxW PseudoR2 t0var1; 
RUN; TITLE;

TITLE "Was Scale-Model RI Variance Significant";
PROC FREQ DATA=Power_Predict_NLMIXED; TABLES tau1sig / BINOMIAL(LEVEL="Yes"); RUN;
TITLE;
